<?php

namespace Drupal\tmgmt_smartling\Tests;

/**
 * Basic flow tests.
 *
 * @group tmgmt_smartling
 */
class BaseFlowTest extends SmartlingTestBase {

  /**
   * Test upload and download translation.
   */
  public function testUploadFileAndDownloadTranslation() {
    if (!empty($this->smartlingPluginProviderSettings)) {
      $translator = $this->setUpSmartlingProviderSettings($this->smartlingPluginProviderSettings);
      $job = $this->requestTranslationForNode($this->testNodeId, $this->targetLanguage, $translator);
      $fileName = $job->getTranslatorPlugin()->getFileName($job);
      $this->checkGeneratedFile($fileName, $this->testNodeTitle);

      // Check fr node title before translation (should be same as en title).
      $this->drupalGet("$this->targetLanguage/node/$this->testNodeId");
      $this->assertResponse(200);
      $this->assertText($this->testNodeTitle);

      // Download translated file.
      $this->downloadAndCheckTranslatedFile($job->id(), $fileName);

      // Check translation.
      $this->drupalGet("$this->targetLanguage/node/$this->testNodeId");
      $this->assertResponse(200);
      $this->assertNoText($this->testNodeTitle);
    }
    else {
      $this->fail("Smartling settings file for simpletests not found.");
    }
  }

  /**
   * Test uploading/downloading xml file: rely on file name in tmgmt_job.
   */
  public function testUploadXmlChangeToXlfDownloadXml() {
    if (!empty($this->smartlingPluginProviderSettings)) {
      $translator = $this->setUpSmartlingProviderSettings($this->smartlingPluginProviderSettings);
      $job = $this->requestTranslationForNode($this->testNodeId, $this->targetLanguage, $translator);
      $fileName = $job->getTranslatorPlugin()->getFileName($job);
      $this->checkGeneratedFile($fileName, $this->testNodeTitle);

      // Check fr node title before translation (should be same as en title).
      $this->drupalGet("$this->targetLanguage/node/$this->testNodeId");
      $this->assertResponse(200);
      $this->assertText($this->testNodeTitle);

      // Change export format to xlf and download translated file.
      $translator->setSetting('export_format', 'xlf');
      $translator->save();
      $this->downloadAndCheckTranslatedFile($job->id(), $fileName);

      // Check translation.
      $this->drupalGet("$this->targetLanguage/node/$this->testNodeId");
      $this->assertResponse(200);
      $this->assertNoText($this->testNodeTitle);
    }
    else {
      $this->fail("Smartling settings file for simpletests not found.");
    }
  }

  /**
   * Test uploading/downloading xlf file: rely on file name in tmgmt_job.
   */
  public function testUploadXlfChangeToXmlDownloadXlf() {
    if (!empty($this->smartlingPluginProviderSettings)) {
      $providerSettings = $this->smartlingPluginProviderSettings;
      $providerSettings['settings[export_format]'] = 'xlf';

      $translator = $this->setUpSmartlingProviderSettings($providerSettings);
      $job = $this->requestTranslationForNode($this->testNodeId, $this->targetLanguage, $translator);
      $fileName = $job->getTranslatorPlugin()->getFileName($job);
      $this->checkGeneratedFile($fileName, $this->testNodeTitle, $providerSettings['settings[export_format]']);

      // Check fr node title before translation (should be same as en title).
      $this->drupalGet("$this->targetLanguage/node/$this->testNodeId");
      $this->assertResponse(200);
      $this->assertText($this->testNodeTitle);

      // Change export format to xml and download translated file.
      $translator->setSetting('export_format', 'xml');
      $translator->save();
      $this->downloadAndCheckTranslatedFile($job->id(), $fileName);

      // Check translation.
      $this->drupalGet("$this->targetLanguage/node/$this->testNodeId");
      $this->assertResponse(200);
      $this->assertNoText($this->testNodeTitle);
    }
    else {
      $this->fail("Smartling settings file for simpletests not found.");
    }
  }

  /**
   * Test revision inflation.
   */
  public function testRevisionInflation() {
    if (!empty($this->smartlingPluginProviderSettings)) {
      $providerSettings = $this->smartlingPluginProviderSettings;
      $translator = $this->setUpSmartlingProviderSettings($providerSettings);
      $job = $this->requestTranslationForNode($this->testNodeId, $this->targetLanguage, $translator);

      $this->drupalGet("/admin/tmgmt/jobs/1");
      $this->assertResponse(200);
      $this->assertNoText("has been accepted as");
      $this->assertNoText("Import of downloaded file was skipped: downloaded and existing translations are equal.");

      $job->getTranslatorPlugin()->downloadTranslation($job);
      $this->drupalGet("/admin/tmgmt/jobs/1");
      $this->assertResponse(200);
      $this->assertText("has been accepted as");
      $this->assertNoText("Import of downloaded file was skipped: downloaded and existing translations are equal.");

      $job->getTranslatorPlugin()->downloadTranslation($job);
      $this->drupalGet("/admin/tmgmt/jobs/1");
      $this->assertResponse(200);
      $this->assertText("Import of downloaded file was skipped: downloaded and existing translations are equal.");
    }
    else {
      $this->fail("Smartling settings file for simpletests not found.");
    }
  }

  /**
   * Test revision inflation with disabled "Auto accept finished translations".
   */
  public function testRevisionInflationAutoAcceptFinishedTranslationsIsDisabled() {
    if (!empty($this->smartlingPluginProviderSettings)) {
      $providerSettings = $this->smartlingPluginProviderSettings;
      $providerSettings['auto_accept'] = FALSE;
      $translator = $this->setUpSmartlingProviderSettings($providerSettings);
      $job = $this->requestTranslationForNode($this->testNodeId, $this->targetLanguage, $translator);

      $this->drupalGet("/admin/tmgmt/jobs/1");
      $this->assertResponse(200);
      $this->assertNoText("is finished and can now be reviewed.");
      $this->assertNoText("Import of downloaded file was skipped: downloaded and existing translations are equal.");

      $job->getTranslatorPlugin()->downloadTranslation($job);
      $this->drupalGet("/admin/tmgmt/jobs/1");
      $this->assertResponse(200);
      $this->assertText("is finished and can now be reviewed.");
      $this->assertNoText("Import of downloaded file was skipped: downloaded and existing translations are equal.");

      $job->getTranslatorPlugin()->downloadTranslation($job);
      $this->drupalGet("/admin/tmgmt/jobs/1");
      $this->assertResponse(200);
      $this->assertText("Import of downloaded file was skipped: downloaded and existing translations are equal.");
      $this->drupalGet("/admin/tmgmt/jobs/1");
    }
    else {
      $this->fail("Smartling settings file for simpletests not found.");
    }
  }

  /**
   * Test request translation when there is one provider in a list.
   *
   * There was an issue when there is only one provider in the list
   * and system requests translation without checkout settings form.
   */
  public function testRequestTranslationWhenOneProviderAvailable() {
    if (!empty($this->smartlingPluginProviderSettings)) {
      $this->drupalPostForm('/admin/tmgmt/translators/manage/smartling', $this->smartlingPluginProviderSettings, t('Save'));
      $this->drupalPostForm('/admin/tmgmt/translators/manage/local/delete', [], t('Delete'));
      $this->drupalPostForm('/admin/tmgmt/translators/manage/file/delete', [], t('Delete'));
      $this->drupalPostForm('/admin/tmgmt/translators/manage/test_translator/delete', [], t('Delete'));

      $this->drupalPostForm('/admin/tmgmt/sources', [
        'items[1]' => 1,
        'target_language' => 'de',
      ], t('Request translation'));

      $this->assertText('One job needs to be checked out.');
      $this->assertText('Create new job');
      $this->assertText('Add to job');
    }
    else {
      $this->fail("Smartling settings file for simpletests not found.");
    }
  }

}
